"use strict";

var babel  = require("babel-core");
var plugin = require("./index");

var assert = require('assert');

function unwrap(code) {
    return code.substring(0, code.length-1);
    return code.substring(15, code.length-1);
}

describe('JSCX', function() {
    it("doesn't touch non-wrapped code", function() {

        this.timeout(10000);

        var code = `<div id="123" />`;

        var output = babel.transform(code, {
            plugins: [plugin, 'syntax-jsx']
            //presets: ['es2015']
        }).code;

        assert.equal(unwrap(output), '<div id="123" />');
    });

    it('converts <cx></cx> to null', function() {

        this.timeout(10000);

        var code = `<cx></cx>`;

        var output = babel.transform(code, {
            plugins: [plugin, 'syntax-jsx']
            //presets: ['es2015']
        }).code;

        assert.equal(unwrap(output), 'null');
    });

   it('allows whitespice within <cx>', function() {

      this.timeout(10000);

      var code = `<cx>
         <Container />
      </cx>`;

      var output = babel.transform(code, {
         plugins: [plugin, 'syntax-jsx']
         //presets: ['es2015']
      }).code;

      assert.deepEqual(eval(output), { type: 'container' });
   });

    it('converts jsx like to config', function() {

        this.timeout(10000);

        var code = `<cx><Component></Component></cx>`;

        var output = babel.transform(code, {
            plugins: [plugin, 'syntax-jsx']
            //presets: ['es2015']
        }).code;

        assert.deepEqual(eval(output), { type: 'component' });
    });

    it('converts jsx attributes to config properties', function() {

        this.timeout(10000);

        var code = `<cx><Component id="123"></Component></cx>`;

        var output = babel.transform(code, {
            plugins: [plugin, 'syntax-jsx']
            //presets: ['es2015']
        }).code;

        assert.deepEqual(eval(output), { type: 'component', id: "123" });
    });

    it('converts jsx literal attributes to config properties', function() {

        this.timeout(10000);

        var code = `<cx><Component id={123}></Component></cx>`;

        var output = babel.transform(code, {
            plugins: [plugin, 'syntax-jsx']
            //presets: ['es2015']
        }).code;

        assert.deepEqual(eval(output), { type: 'component', id: 123 });
    });

    it('converts jsx object attributes to config properties', function() {

        this.timeout(10000);

        var code = `<cx><Component value={{ x: 1 }}></Component></cx>`;

        var output = babel.transform(code, {
            plugins: [plugin, 'syntax-jsx']
            //presets: ['es2015']
        }).code;

        assert.deepEqual(eval(output), { type: 'component', value: { x: 1 } });
    });

    it('converts jsx function attributes to config properties', function() {

        this.timeout(10000);

        var code = `<cx><Component onClick={e=>{}}></Component></cx>`;

        var output = babel.transform(code, {
            plugins: [plugin, 'syntax-jsx', 'transform-es2015-arrow-functions']
            //presets: ['es2015']
        }).code;

        assert.deepEqual(typeof eval(output).onClick, "function");
    });

    it('converts jsx children into children array', function() {
        this.timeout(10000);

        var code = `<cx><Component><Child><GrandChild></GrandChild></Child></Component></cx>`;

        var output = babel.transform(code, {
            plugins: [plugin, 'syntax-jsx']
            //presets: ['es2015']
        }).code;

        assert.deepEqual(eval(output), {
            type: 'component',
            children: [{type: 'child', children: [{type: 'grandchild'}]}]
        });
    });

   it('converts jsx text child into text widget', function() {
      this.timeout(10000);

      var code = `<cx><Component>Text</Component></cx>`;

      var output = babel.transform(code, {
         plugins: [plugin, 'syntax-jsx']
         //presets: ['es2015']
      }).code;

      assert.deepEqual(eval(output), {
         type: 'component',
         children: ['Text']
      });
   });

   it('converts dot attributes into object configs', function() {
      this.timeout(10000);

      var code = `<cx><Component value:bind="name" /></cx>`;

      var output = babel.transform(code, {
         plugins: [plugin, 'syntax-jsx']
         //presets: ['es2015']
      }).code;

      assert.deepEqual(eval(output), {
         type: 'component',
         value: {bind: "name"}
      });
   });

   it('multiple root elements are converted to a config array', function() {
      this.timeout(10000);

      var code = `<cx><Component /><Component2></Component2></cx>`;

      var output = babel.transform(code, {
         plugins: [plugin, 'syntax-jsx']
         //presets: ['es2015']
      }).code;

      assert.deepEqual(eval(output), [{type: 'component'}, {type: 'component2'}]);
   });
});
//